package com.astateofapps.crawler;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Set;

public class DomainCrawler {

    private static final String TASK_URL = "http://stackoverflow.com/questions/5834808/designing-a-web-crawler";
    private static final int TIME_FOR_WORK_IN_MILLIS = 120000;//2 min

    private Set<String> visitedDomains;

    public DomainCrawler() {
        this.visitedDomains = new HashSet<String>();
    }

    /**
     * WebCrawler test
     *
     * @param millis time for test
     * @param url    root page
     * @return number of unique domains
     * @throws IOException
     */
    public int testCrawler(int millis, String url) throws IOException {
        long start = System.currentTimeMillis();
        long end = start + millis;
        while (System.currentTimeMillis() < end) {
            processPage(url, end);
        }
        return visitedDomains.size();
    }

    public static void main(String[] args) throws IOException {
        DomainCrawler crawler = new DomainCrawler();
        int size = crawler.testCrawler(TIME_FOR_WORK_IN_MILLIS, TASK_URL);
        System.out.println("Total size: " + size);
    }

    /**
     * Scans the page for new domains
     *
     * @param url root page
     * @param end time, to end crawling
     * @throws IOException
     */
    private void processPage(String url, long end) throws IOException {
        if (System.currentTimeMillis() >= end) {// for testing purposes
            return;
        }

        String domain = getDomainName(url);
        if (domain != null) {
            visitedDomains.add(domain);
            System.out.println(domain);
        } else {
            return;
        }
        if (url.contains(".pdf")
                || url.contains(":80")
                || url.contains(".jpg")
                || url.contains(".png")
                || url.contains(".pdf")
                || url.contains(".gif")
                || url.contains(".rar")
                || url.contains(".zip")
                || url.contains("#"))
            return;
        try {
            Document doc = Jsoup.connect(url).get();
            Elements questions = doc.select("a[href]");

            for (Element link : questions) {
                String hrefUrl = link.attr("abs:href");
                String hrefDomain = getDomainName(hrefUrl);
                if (hrefDomain != null && !visitedDomains.contains(hrefDomain)) {
                    processPage(hrefUrl, end);
                }
            }
        } catch (Exception e) {
            System.out.println("Can't handle " + url);
        }
    }

    /**
     * Takes domain from url
     *
     * @param url String with url
     * @return domain
     */
    private static String getDomainName(String url) {
        URI uri;
        try {
            uri = new URI(url);
        } catch (URISyntaxException e) {
            return "";
        }
        String domain = uri.getHost();
        domain = domain.startsWith("www.") ? domain.substring(4) : domain;
        return domain;
    }
}

